% ########################################################################
%  ATTENTION
% ########################################################################
%
%  This software is for guidance only. It is provided "as is".
%  It only aims at providing coding information to the customer in order
%  for them to save time. As a result no warranties, whether express, 
%  implied or statutory, including, but not limited to implied warranties 
%  of merchantability and fitness for a particular purpose apply 
%  to this software.
%  A·P·E Angewandte Physik und Elektronik GmbH shall not be held liable 
%  for any direct, indirect or consequential damages with respect to any 
%  claims arising from the content of such sourcecode and/or the use made 
%  by customers of the coding information contained herein in connection
%  with their products.
%
%  File Version: 1.0
%
%  Changelog:
%    1.0 - Initial Version
%
% ########################################################################

classdef ape_device
	% APEDevice class
    
	properties
		dev; 					% TCP Connection object
		dev_in;					% TCP Stream reader
		dev_out;				% TCP Stream writer
		host = 'localhost';		% Hostname or IP-address
		port = 51019;			% TCP Port
		timeout = 3.0;			% Timeout for tcp read/write actions
	end

	methods

% CONSTRUCTOR function ---------------------------------------------------
		function obj = ape_device(host,port, timeout)
        % APE_DEVICE Creates and opens a tcp-connection to the target
        %
        %  myDevice = APE_DEVICE(hostname, port, timeout)
           
            
			% import java tcp socket libraries
			import java.io.*;
			import java.net.*;

			% class constructor
			if(nargin > 0)
					obj.host = host;
					obj.port = port;
					obj.timeout = timeout;
     		end	

     		% open Socket connection to remote target
			try				
				address = InetSocketAddress(host, port);

				obj.dev = Socket();
				obj.dev.connect(address, timeout);

				% wait until the socket becomes ready
				pause(2.0)

				% create stream-reader and -writer
				obj.dev_in =  DataInputStream(obj.dev.getInputStream);
                obj.dev_out = DataOutputStream(obj.dev.getOutputStream);
			
			% An error occured during connection setup 
            % -> stop socket + I/Os if they exist
			catch ME
				warning('Socket initialization failed!');

				if ~isempty(obj.dev) 
					obj.dev.close();
					clear obj.dev;
				end
				if ~isempty(obj.dev_in)
					obj.dev_in.close();
					clear obj.dev_in;
				end
				if ~isempty(obj.dev_out)
					obj.dev_out.close();
					clear obj.dev_out
				end
				rethrow(ME);
			end
		end

% CLOSE function ---------------------------------------------------------
		function close(obj)
			% CLOSE Closes all existing connections for this ape_device
			% object
            %
            % CLOSE()

			if ~isempty(obj.dev)
				obj.dev.close();
			end

			if ~isempty(obj.dev_in)
				obj.dev_in.close();
			end

			if ~isempty(obj.dev_out)
				obj.dev_out.close();
			end
		end

% WRITE function ---------------------------------------------------------
		function write(obj, data)
            % WRITE Writes the given data to the tcp-connection
            %
            % WRITE(data)     

            if ~exist('data')
                error('WRITE: Data must not be empty!')
            else
                cmd = sprintf('%s\r\n', data);
                obj.dev_out.writeBytes(cmd);
                obj.dev_out.flush();
            end
		end

% READ LINE function -----------------------------------------------------
		function ans = readLine(obj, nr_bytes)
        % READLINE Reads data from the socket until a newline-character is read
        % or the specified number of bytes (1b = 1 character) was read
        %
        % buffer = READLINE() Reads up tp 2048 characters from the socket
        % until a newline (\n) character was read.
        %
        % buffer = READLINE(nr_bytes) Reads nr_bytes chacacters from the
        % socket until a newline (\n) character was read.
            
            
			% set max number of chars to read to prevent buffer overflows
			if ~exist('nr_bytes')
				nr_bytes = 2048;
            end
            
			ans = []; % return buffer
			x = 0; % loop cycle counter
            
            % read single characters from the socket until
            % nr_bytes bytes were read
                      
            while(1)
                
                c = obj.dev_in.read();

                %append byte to answer array
                ans = [ans c];

                x = x+length(c);

                %check for endline character                
                if ( ~isempty(find(c==sprintf('\n'),1)) )
                    break;
                end      

                % if max. number of chars is reached, 
                % stop reading new bytes
                if (x >= nr_bytes)
                    break;
                end
            end
        end

% READ function ----------------------------------------------------------
		function ans = read(obj, nr_bytes)
        % READ Reads data from the target up to the specified number of
        % bytes
        %
        % buffer = READ(nr_bytes)

			ans = '';

			if (nr_bytes < 0)
				ans = obj.dev_in.readLine();
			else
				x = 0; % loop cycle counter
                
                % read single characters from the socket until
				% nr_bytes bytes were read        
                while(1)

                    c = obj.dev_in.read();
                    % append byte to answer array
                    ans = [ans c];

                    x = x+length(c);   

                    % if max. number of chars is reached, 
                    % stop reading new bytes
                    if (x >= nr_bytes)
                        break;
                    end
                end
            end
		end

% READ BLOCK function ----------------------------------------------------
		function ans = readBlock(obj)
        % READBLOCK Reads a SCPI block from the socket and unpacks 
        % the contained data
        %
        % buffer = READBLOCK()

			ans = '';			

			% read SCPI Block Start-Character
			c = char(obj.read(1));
			if ~strcmp(c,'#')
				warning('Invalid SCPI Block segment received');
			else
				% Get number of characters in header length
				header_length = str2num(obj.read(1));
				if (header_length < 0) || (header_length > 9)
					warning('SCPI Block Header Length invalid');
				else
					%Get number of bytes in data segment
					data_length = str2num(obj.read(header_length));
					if (data_length < 0) || (data_length > 999999999)
						warning('SCPI Block Data Length invalid');
					else
						% Read SCPI Block data
						ans = obj.read(data_length);
					end
				end
			end
		end

% QUERY function ---------------------------------------------------------
		function ans = query(obj, command, read_block)
        % QUERY Writes a command to the remote target and reads back
        % the answer generated by the target
        %
        % buffer = QUERY(command) Writes the command to the target and
        % reads back a single line of characters
        %
        % buffer = QUERY(command, true) Writes the command to the
        % target and tries to read back an SCPI block

			% set read_block to false, if not set in function call
			if ~exist('read_block')
				read_block = false;
			end

			if ~exist('command')
				error('Command string must not be empty!')
			else
				% write command to the remote device
				obj.write(command);

				% select if a single line or a scpi block should be 
				% read from the socket
				if read_block
					ans = obj.readBlock();
				else
					ans = char(obj.readLine());
				end
			end
		end

% IDN function -----------------------------------------------------------
		function ans = idn(obj)
			% IDN Queries the device identification string (*IDN)
            %
            % buffer = IDN()
			ans = obj.query('*idn?');
		end
	end
end
